/*

  Plot a graph of some points, with (optionally) error bars.

  I've found various programs lying about that plot graphs.
  Unfortunately none of them cope with error bars, which is
  why I'm doing this.


  Input is taken from standard in.  Format is

       Layout
       Title
       Horizontal axis range
       Vertical axis range
       {
       Points
       }

  where

  Layout         =  The axis box.
		    Positioning of title.
		    Size of title text.
		    For horizontal and vertical axes:
			Size of gradations, major and minor.
			Positioning of numbers relative to major gradations.
			Size of number text.
			Positioning of axis labels relative to ends of axes.
			Size of label text.
		    Width of points.


  Title

  Axis range      = Extrema,
                    Numbering "anchor" and interval,
		    Num minor gradations per interval, 
		    Numbering format, label
		    Whether axis duplicated on both sides of box.

  Points          = Format of points, Whether to do spline interpolation,
                    Whether error bars,
		    Whether abscissae implicitly calculable (because
		    constant intervals).  If so, parameters.

		    Data

  All dependence of the screen coordinate system is localised in the
  "layout".  At the moment I'm not going to specify what the screen
  coordinate system actually looks like.  And the program should 
  incorporate enough data abstaction to postpone this decision
  as long as possible.

  Axes have major and minor gradations.  Major gradations are numbered,
  minor gradations are not.  The positioning of major gradations is
  defined by an "anchor" and an interval.  The anchor is any point at
  which there is to be a gradation, or would be a gradation if the
  point were within the extrema of the axis range (it can be outside).
  Major gradations are filled in on either side of the anchor every +ve
  or -ve integer multiple of 'interval'.

  There will be several predefined formats (shapes and colours) for
  points, including invisible (useful if a spline interpolation is
  required).

  "data" will be a series of records, each defining one point.
  The numbers in the record will have "float" format.  The number
  of numbers per point depends on whether abscissae or ordinates
  have error bars, and on whether abscissae are implicitly
  calculated.  The data list is terminated by some special
  record.  See next paragraph.

  A friendly version of this program would allow fairly free
  input format, and all the control records would have
  command names, and parameter fields would be named, etc.  Since
  I'm not really in the business of providing a friendly program I'm
  going to throw all this stuff out of the window, and go for
  a fixed format input, with meanings defined by position, and
  nearly all fields compulsory, and so on.  This will avoid
  all that tedious parsing and error reporting stuff.
  I suppose I should try and arrange things so that that stuff
  could be put on top later.

  I haven't yet decided what the best target language is
  (fig file, pic file, postscript file,...).  For development
  I'll probably generate a fig file, but this should be
  abstracted as much as possible.

*/

/*

  Comments on existing version...

  1) I made a big mistake in mapping direct from the user coordinates
     to screen coordinates (at the fig level).  I should have gone 
     through an intermediate stage of floating point screen coordinates.
     This would have eliminated some annoying inaccuracies in the
     plotting of axis gradations due to truncation, and permitted more 
     accuracy in the definition of shapes for the plot points,
     as well as just adding another layer of abstraction.

  2) The data structures need rearranging a bit.

     The idea of separate layout and range structures for axes is
     probably ok.  But the definition of the screen location of
     the axes is in a completely stupid place - not even in
     the axis layout structure - instead given as bottom-left,
     top-right coordinates elsewhere.  This caused some big
     headaches.

  3) This is a minor point which only affects the bottom level output
     routines: at the moment text is left justified relative to
     the specified position.  It would be better if it was centred,
     but I haven't found out how to specify this in the fig
     representation.

  4) Extras which would be nice...

     . A friendly format for input.
     . The ability to put a key of point shapes somewhere on the graph.

*/


#include "types.h"

/* 'layout', 'x_axis' and 'y_axis' defined in 'plot.c' */

extern LAYOUT layout ;
extern AXS_RANGE x_axis, y_axis ;

void main (argc, argv)
int argc ;
char *argv [] ;
{
  char title [MAX_STR_SIZE] ;
  DAT_HDR data_header ;
  float data_point [MAX_PNT_SIZE] ;

  get_layout (&layout) ;
  get_title  (title)   ;
  get_axis   (&x_axis) ;
  get_axis   (&y_axis) ;

  plot_box   (title)   ;

  while (get_data_header (&data_header))
  {
    plot_begin (&data_header) ;
    while (get_point (data_point))
      plot_point (data_point) ;
    plot_end   (&data_header) ;
  }
}
